/*
Copyright (C) 2012 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
 */
package org.harctoolbox.IrpMaster;

/**
 * This class contains some JP1 related computations, in the form of static functions.
 *
 */
public class EFC {
    public static int efc2hex(int efc) {
        int temp = efc + 156;
        temp = (temp & 0xFF) ^ 0xAE;
        return (( temp >> 3 ) | ( temp << 5 )) & 0xFF;
    }

    public static int hex2efc(int hex) {
        int rc = hex & 0xFF;
        rc = (rc << 3) | (rc >> 5);
        rc = (rc ^ 0xAE) - 156;
        return rc & 0xFF;
    }

    public static int hex2efc5(int hex, int no_bytes) {
        if (no_bytes == 2) {
            int byte1 = (hex >> 8) & 0xFF;
            byte1 ^= 0x00D5;
            byte1 = (byte1 >> 5 | byte1 << 3) & 0xFF;
            byte1 = byte1 - 100 & 0xFF;
            int byte2 = (hex) & 0xFF ^ 0xC5;
            int rc = (byte1 << 8) + byte2;
            if (rc < 1000) {
                rc += 65536;
            }
            return rc & 0xFFFF;
        } else {
            return hex2efc(hex);
        }
    }

    public static int efc52hex(int val, int no_bytes) {
        if (no_bytes == 1) {
            return efc2hex(val & 0xFF);
        } else {
            int byte1 = (short) (val >> 8 & 0x00FF);
            byte1 += 100;
            byte1 &= 0xFF;
            byte1 = byte1 << 5 | byte1 >> 3;
            byte1 ^= 0x00D5;
            int data0 = byte1 & 0x00FF;
            int data1 = val & 0x00FF ^ 0x00C5;
        return (data0 << 8) + data1;
        }
    }

    private static void main(String[] args) {
        System.out.println("testing hex2efc(efc2hex(i))");
        for (int i = 0; i < 256; i++) {
            int error = i - hex2efc(efc2hex(i));
            if (error != 0)
                System.out.println(i + "\t" + efc2hex(i) + "\t" + hex2efc(efc2hex(i)));
        }
        System.out.println("testing efc2hex(hex2efc(i)");
        for (int i = 0; i < 256; i++) {

            int error = i - efc2hex(hex2efc(i));
            if (error != 0)
                System.out.println(i + "\t" + efc52hex(i, 1) + "\t" + hex2efc5(efc52hex(i, 1), 1));
        }
        for (int i = 0; i < 256; i++) {
            int error = i - hex2efc5(efc52hex(i, 1), 1);
            if (error != 0)
                System.out.println(i + "\t" + efc52hex(i, 1) + "\t" + hex2efc5(efc52hex(i, 1), 1));
        }
        for (int i = 0; i < 256; i++) {
            int error = i - efc52hex(hex2efc5(i, 1), 1);
            if (error != 0)
                System.out.println(i + "\t" + efc52hex(i, 1) + "\t" + hex2efc5(efc52hex(i, 1), 1));
        }
        for (int i = 0; i < 65536; i++) {
            int error = i - hex2efc5(efc52hex(i, 2), 2);
            if (error != 0)
                System.out.println(i + "\t" + efc52hex(i, 2) + "\t" + hex2efc5(efc52hex(i, 2), 2));
        }
        for (int i = 0; i < 65536; i++) {
            int error = i - efc52hex(hex2efc5(i, 2), 2);
            if (error != 0)
                System.out.println(i + "\t" + efc52hex(i, 1) + "\t" + hex2efc5(efc52hex(i, 1), 1));
        }
    }
}
