/*
Copyright (C) 2012 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
 */

package org.harctoolbox.IrpMaster;

import java.io.File;
import java.io.PrintStream;

/**
 * This file generates a Lintronic export of an IrSequence.
 *
 * <p>The class was written by looking at the MakeLintronic function in Graham Dixon's library ExchangeIR,
 * but the code herein has little in common with Graham's code.
 */
public class Lintronic {
    
    // explicitly prohibit construction
    private Lintronic() {
    }         

    /**
     * Generates a Lintronic export according to the argument. The system's native line separator is used
     * to separate the lines in the generated String.
     *
     * @param irSequence IrSequence to export
     * @return export as String.
     */
    public static String toExport(IrSequence irSequence) {
        String lineSeparator = System.getProperty("line.separator", "\n");
        String result = "Pulse, Pause in micro-seconds" + lineSeparator;

        for (int i=0; i<irSequence.getNumberBursts(); i++) {
            long t1 = Math.min(Math.round(Math.abs(irSequence.get(2*i))), 99999L);
            long t2 = Math.min(Math.round(Math.abs(irSequence.get(2*i+1))), 99999L-t1);
            result = result + String.format("%05d,%05d = %05d%s", t1, t2, t1+t2, lineSeparator);
	}
        return result;
    }

    /**
     * Just for testing.
     * @see IrSignal
     * @param args
     */
    public static void main(String[] args) {
        String protocolsIni = "data/IrpProtocols.ini";
        int arg_i = 0;
        if (args[arg_i].equals("-c")) {
            arg_i++;
            protocolsIni = args[arg_i++];
        }
        try {
            IrSignal irSignal = new IrSignal(protocolsIni, arg_i, args);
            System.out.println(irSignal);
            //System.out.println(irSignal.ccfString());
            DecodeIR.invoke(irSignal);
            PrintStream ps = new PrintStream(new File("lintronic.txt"));
            ps.print(toExport(irSignal.toModulatedIrSequence(6)));
            ps.close();
        } catch (Exception ex) {
            System.err.println(ex.getMessage());
        }
    }
}
